# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-11-02 09:50
from __future__ import unicode_literals

import contentstore.models
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contentstore', '0006_messageset_channel'),
    ]

    operations = [
        migrations.AlterField(
            model_name='message',
            name='text_content',
            field=models.TextField(blank=True, null=True, validators=[contentstore.models.validate_special_characters]),
        ),
    ]
