# -*- coding: utf-8 -*-
# Generated by Django 1.9.11 on 2017-01-16 07:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0003_auto_20160322_1534'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='initial_sequence_number',
            field=models.IntegerField(default=1),
        ),
    ]
