# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-07-27 08:20
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('subscriptions', '0005_subscriptionsendfailure'),
    ]

    operations = [
        migrations.AlterField(
            model_name='subscription',
            name='identity',
            field=models.CharField(db_index=True, max_length=36),
        ),
    ]
