# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-04-19 06:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contentstore', '0005_auto_20160912_0923'),
    ]

    operations = [
        migrations.AddField(
            model_name='messageset',
            name='channel',
            field=models.CharField(blank=True, max_length=64, null=True, verbose_name='Channel'),
        ),
    ]
